{% extends "touchtechnology/news/base.html" %}
{% load i18n %}
{% load content %}

{% block body_class %}{{ block.super }} feeds{% endblock %}

{% block news_content %}
	<h1>{% block page_title %}{% trans "Feeds" %}{% endblock %}</h1>

	{% block syndication %}
		{% chunk "syndication-info" %}
	{% endblock %}

	{% block syndication_terms %}
		{% chunk "syndication-terms" %}
	{% endblock %}

	<ul>
		{% for name, feed in feeds.items %}
			<li><a href="{% url application.name|add:":feed" feed %}">{{ node.title }} ({{ name }})</a></li>
		{% endfor %}
	</ul>
{% endblock %}
