{% extends "touchtechnology/content/admin/delete_confirm.html" %}

{% block cancel_url %}{% if path %}{% url 'admin:files:listdir' path %}{% else %}{% url 'admin:files:index' %}{% endif %}{% endblock %}
